#include <Eigen/Core>
#include <iostream>
#include "writer.hpp"
#include <cmath>
#include <stdexcept>

//! Vector type
typedef Eigen::VectorXd Vector;

/// Uses forward Euler and upwind finite differences to compute the solution u to Burgers' equation from time 0 to time T 
///
/// @param[out] u solution at all time steps up to time T, each column corresponding to a time step (including the initial condition as first column)
/// @param[out] time the time levels
/// @param[in] u0 the initial conditions, as column vector
/// @param[in] dt the time step size
/// @param[in] T the final time at which to compute the solution
/// @param[in] N the number of grid points, INCLUDING BOUNDARY POINTS
///

void BurgersUpwind(Eigen::MatrixXd & u,  Vector & time, const Vector u0, double dt, double T, int N)
{
  const unsigned int nsteps = round(T/dt);
  const double dx = 1./(N-1);
  u.resize(N,nsteps+1);
  time.resize(nsteps+1);

  /* Initialize u */


  time[0]=0;

  /* Implement timestepping */


}

/* Initial condition 1 */
double U0_s(double x) { 
  return std::max(1.-2.*x,0.);
}

/* Initial condition 2 */
double U0_r(double x) { 
  if(x <= 1./2.)
	return 0;
  else
	return 1;
}

int main(int, char**) {
    double T = 2.;
    double dt = 0.001; 
    int N=101;
    Vector u0(N);
    double dx=1./(N-1);
    /* Initialize u0 */
    for(int i=0;i<u0.size();i++)
      u0[i] = U0_s(dx*(double)i);

    Eigen::MatrixXd u_burgers1;
    Vector time1;
    BurgersUpwind(u_burgers1,time1,u0,dt,T,N);
    writeToFile("time1.txt",time1);
    writeMatrixToFile("u_burgers1.txt", u_burgers1);

    /* Reinitialize u0 */
    for(int i=0;i<u0.size();i++)
      u0[i] = U0_r(dx*(double)i);
    Eigen::MatrixXd u_burgers2;
    Vector time2;
    BurgersUpwind(u_burgers2,time2,u0,dt,T,N);
    writeToFile("time2.txt",time2);
    writeMatrixToFile("u_burgers2.txt", u_burgers2);
}
